#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "json-main.h"

#include <iostream>
#include <sys/un.h>
#include <sys/socket.h>
#include <unistd.h>


// Analog of main-window.cc
// TODO: Think more about concurrency and null pointer exceptions

void* run_loop_pthread(void* arg) {
  json_main* _json_main = static_cast<json_main*>(arg);
  _json_main->run_loop();
  return NULL;
}

void json_object_cb(std::string name, JSON_OBJECT_T jobj, void* arg) {
  json_main* _json_main = static_cast<json_main*>(arg);
  _json_main->process_json_object(name, jobj);
}

json_main::json_main(const std::string& json_sock_path)
  : _json_sock_path (json_sock_path),
    _loop_thread_active (false),
    _octave_json_link (this)
{
  // Enable octave_json_link instance
	octave_link::connect_link(&_octave_json_link);

  // Open UNIX socket file descriptor
  sockfd = socket(AF_UNIX, SOCK_STREAM, 0);
  struct sockaddr_un addr;
  memset(&addr, 0, sizeof(addr));
  addr.sun_family = AF_UNIX;
  memcpy(&addr.sun_path, _json_sock_path.c_str(), sizeof(addr.sun_path)-1);
  connect(
    sockfd,
    reinterpret_cast<struct sockaddr*>(&addr),
    sizeof(addr));
}

json_main::~json_main(void) {
  close(sockfd);

  // TODO: Stop the _loop_thread
}

void json_main::publish_event(const std::string& name, JSON_OBJECT_T jobj) {
  std::string jstr = json_util::to_message(name, jobj);
  send(sockfd, jstr.c_str(), jstr.length(), 0);
}

void json_main::run_loop_on_new_thread(void) {
  if (_loop_thread_active)
    perror("won't run JSON socket loop multiple times");
  _loop_thread_active = true;

  pthread_create(
    &_loop_thread,
    NULL,
    run_loop_pthread,
    static_cast<void*>(this));
}

void json_main::run_loop(void) {
  json_util::read_stream(
    sockfd,
    json_object_cb,
    static_cast<void*>(this));
}

void json_main::process_json_object(std::string name, JSON_OBJECT_T jobj) {
  _octave_json_link.receive_message(name, jobj);
}
